dnl
dnl acinclude.m4 - configuration macros for configure.in
dnl 
dnl Copyright (C) 2005  Toni Ronkko
dnl 
dnl Peach Gardens is free software; you can redistribute it and/or modify 
dnl it under the terms of the GNU General Public License as published by the
dnl Free Software Foundation; either version 2 of the License, or (at your
dnl option) any later version.
dnl
dnl Peach Gardens is distributed in the hope that it will be useful, but 
dnl WITHOUT ANY WARRANTY; without even the implied warranty of 
dnl MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU 
dnl General Public License for more details.
dnl
dnl You should have received a copy of the GNU General Public License along
dnl with Peach Gardens; if not, write to the Free Software Foundation, Inc.,
dnl 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
dnl 
dnl 
dnl Nov 2004, Toni Ronkko <toni at users.playstation2-linux.com>
dnl
dnl $Id: acinclude.m4,v 1.3 2005/06/22 13:03:19 tronkko Exp $
dnl
dnl $Log: acinclude.m4,v $
dnl Revision 1.3  2005/06/22 13:03:19  tronkko
dnl maemo and ps2 are check only when --with-maemo or --with-ps2 is given
dnl in command line
dnl
dnl Revision 1.2  2005/01/09 21:17:10  tronkko
dnl updated copyright
dnl
dnl Revision 1.1  2004/11/20 10:59:05  tronkko
dnl Initial revision
dnl
dnl 
dnl

dnl
dnl Allow user to compile both debug and release versions of the 
dnl application.  The macro works by defining `DEBUG' conditional to be 
dnl used in Makefile.am and by defining `NDEBUG' variable to be used in 
dnl C code.  The `DEBUG' and `NDEBUG' macros together allow you to insert 
dnl conditional debug options into your Makefiles and enable more verbose 
dnl output in C-code.
dnl
dnl Cached variables:
dnl   pg_cv_debug
dnl   pg_cv_debug_cflags
dnl
dnl Conditionals defined:
dnl   DEBUG
dnl   
AC_DEFUN([ENABLE_DEBUG],[
  AC_REQUIRE([AC_PROG_CC])
  pg_cv_debug_cflags=""

  dnl let user specify --enable-debug in command line
  AC_ARG_ENABLE(debug,
  [  --enable-debug          enable internal sanity checks],
  [case "${enableval}" in
    yes) pg_cv_debug=true ;;
    no)  pg_cv_debug=false ;;
    *) AC_MSG_ERROR(bad value ${enableval} for --enable-debug) ;;
  esac],[pg_cv_debug=false])

  dnl check compiler flags
  if test "x$pg_cv_debug" = xtrue; then
    dnl save compiler environment
    AC_LANG_SAVE
    pg_debug_cflags="$CFLAGS"
    pg_cv_debug_cflags="$CFLAGS"

    dnl try appending some debugging flags to command line (C)
    AC_MSG_CHECKING(for extra debug options)
    AC_LANG_C
    CFLAGS="-g3 -O0 -Waggregate-return -Wall -W -Wpointer-arith -Wcast-align -Wwrite-strings -Wmissing-prototypes -Wmissing-declarations -fno-writable-strings"
    AC_TRY_COMPILE([#include <stdio.h>], [printf("hello world\n");],
      pg_cv_debug_cflags="$CFLAGS")
    AC_MSG_RESULT($pg_cv_debug_cflags)

    dnl restore compiler environment
    CFLAGS="$pg_debug_cflags"
    AC_LANG_RESTORE
  fi
  
  dnl Add debugging flags to command line.  Note that user's options must
  dnl be added after the debug options so that the user could undo some of 
  dnl the options, should he want that.  The feature can be for example used
  dnl to undefine NDEBUG (-UNDEBUG in gcc) to enable assertions on otherwise
  dnl optimized executable.
  if test x$pg_cv_debug = xtrue; then
    CFLAGS="$pg_cv_debug_cflags $CFLAGS"
  else
    CFLAGS="-DNDEBUG $CFLAGS"
  fi

  dnl define conditionals for automake
  AM_CONDITIONAL(DEBUG, test x$pg_cv_debug = xtrue)
])


dnl
dnl Compile for Nokia 770 Internet Tablet.  Option --with-maemo must be 
dnl specified in command line to even consider this option.
dnl
dnl Cached variables:
dnl   pg_cv_maemo
dnl
dnl Compiler defines affected:
dnl   MAEMO
dnl
dnl Automake variables affected:
dnl   @GFXLIBS@
dnl
AC_DEFUN([LIBMAEMO],[

  AC_ARG_WITH(maemo,
    [  --with-maemo            compile for Nokia 770 Internet Tablet],
    [with_maemo="$withval"],
    [with_maemo="no"])

  if test x"$with_maemo" = x"yes"; then
    dnl save environment
    pg_maemo_libs="$LIBS"
    pg_maemo_cflags="$CFLAGS"
    AC_LANG_SAVE

    dnl get target
    OSSI_TARGET_BOARD=`echo $OSSI_TARGET_BOARD`
    AC_SUBST(OSSI_TARGET_BOARD)

    dnl check for required libs (taken from maemopad sample)
    PKG_CHECK_MODULES(HILDON, hildon-libs >= 0.8.4)
    PKG_CHECK_MODULES(MB, libmb >= 0.8)
    MB_PREFIX=`pkg-config libmb --variable=prefix`
    PKG_CHECK_MODULES(OSSO,libosso >= 0.8.4 gnome-vfs-2.0 >= 2.2 gnome-vfs-module-2.0 >= 2.2)
    PKG_CHECK_MODULES(OSSOSETTINGS, osso-af-settings >= 0.8.4)

    dnl compile and link a maemo application to see if above tests failed
    AC_MSG_CHECKING(for Maemo)
    AC_LANG_C
    LIBS="$LIBS $HILDON_LIBS $MB_LIBS $OSSO_LIBS"
    CFLAGS="$CFLAGS $HILDON_CFLAGS $MB_CFLAGS $OSSO_CFLAGS"
    AC_TRY_LINK([
#include <hildon-widgets/hildon-app.h>
#include <gtk/gtkmain.h> ], 
      [hildon_app_set_killable ((HildonApp*)0, 0);],
      pg_cv_maemo=yes,
      pg_cv_maemo=no)
    AC_MSG_RESULT($pg_cv_maemo)

    dnl restore environment
    LIBS="$pg_lmaemo_libs"
    CFLAGS="$pg_maemo_cflags"
    AC_LANG_RESTORE

    dnl insert -DMAEMO to command line if the library was found
    if test "x$pg_cv_maemo" = xyes; then
      CFLAGS="$CFLAGS -DMAEMO $HILDON_CFLAGS $MB_CFLAGS $OSSO_CFLAGS"

      dnl export libs to Makefiles
      GFXLIBS="$HILDON_LIBS $MB_LIBS $OSSO_LIBS $GFXLIBS"
      AC_SUBST(GFXLIBS)

      dnl export MB_PREFIX to Makefiles
      AC_SUBST(MB_PREFIX)

      dnl installation directories
      localedir=`$PKG_CONFIG osso-af-settings --variable=localedir`
      hildondesktopentrydir=`$PKG_CONFIG osso-af-settings --variable=desktopentrydir`
      hildoncpdesktopentrydir=`$PKG_CONFIG hildon-control-panel --variable plugindesktopentrydir`
      hildonstatusbarplugindir=`$PKG_CONFIG hildon-status-bar-lib --variable=pluginlibdir`
      hildoncplibdir=`$PKG_CONFIG hildon-control-panel --variable=pluginlibdir`
      hildoniconthemedir=$HOME/.icons/hicolor

      AC_SUBST(localedir)
      AC_SUBST(hildondesktopentrydir)
      AC_SUBST(hildoncpdesktopentrydir)
      AC_SUBST(hildonstatusbarplugindir)
      AC_SUBST(hildoncplibdir)
      AC_SUBST(hildoniconthemedir)
      AC_SUBST(desktopentrydir)
    fi
  else
    AC_MSG_CHECKING(for Maemo)
    AC_MSG_RESULT(disabled)
  fi

  dnl define conditional for Makefiles
  AM_CONDITIONAL(MAEMO, test "x$pg_cv_maemo" = xyes)
])


dnl
dnl Compile for PlayStation 2 Linux Kit.  Option --with-ps2 must be 
dnl specified in command line of configure to even consider this option.
dnl
dnl Cached variables:
dnl   pg_cv_libps2dev
dnl
dnl Compiler defines affected:
dnl   PS2DEV
dnl
dnl Automake variables affected:
dnl   @GFXLIBS@
dnl
AC_DEFUN([LIBPS2DEV],[
  pg_cv_libps2dev=no
  AC_ARG_WITH(ps2,
    [  --with-ps2              compile for PlayStation 2 Linux kit],
    [with_ps2="$withval"],
    [with_ps2="no"])

  if test x"$with_ps2" = xyes; then

    dnl save environment
    pg_libps2dev_libs="$LIBS"
    AC_LANG_SAVE

    dnl check for libps2dev
    AC_MSG_CHECKING(for libps2dev)
    AC_LANG_C
    LIBS="$LIBS -lps2dev"
    AC_TRY_LINK([
#include <ps2gs.h>
#include <ps2dma.h>], 
      [ps2_gs_vc_graphicsmode();],
      pg_cv_libps2dev=yes,
      pg_cv_libps2dev=no)
    AC_MSG_RESULT($pg_cv_libps2dev)

    dnl restore environment
    LIBS="$pg_libps2dev_libs"
    AC_LANG_RESTORE

    dnl insert -DPS2DEV to command line if the library was found.
    if test "x$pg_cv_libps2dev" = xyes; then
      CFLAGS="$CFLAGS -DPS2DEV"
      GFXLIBS="-lps2dev $GFXLIBS"
      AC_SUBST(GFXLIBS)
    fi
  else
    AC_MSG_CHECKING(for libps2dev)
    AC_MSG_RESULT(disabled)
  fi

  dnl define conditional for Makefiles
  AM_CONDITIONAL(PS2DEV, test "x$pg_cv_libps2dev" = xyes)
])


dnl
dnl Compile for X11.  This is the default target: option --without-x 
dnl must be specified in command line if X11 compilation is not wished.
dnl
dnl Cached variables:
dnl   pg_cv_libs_x11, pg_cv_xf86
dnl
dnl Compiler defines affected:
dnl   X11, XF86VMODE
dnl
dnl Automake variables affected:
dnl   @GFXLIBS@
dnl
AC_DEFUN([LIBX11],[
  pg_cv_libs_x11=no

  if test x"$have_x" = x"yes"; then
    dnl Do consider compiling for X11
    AC_REQUIRE([AC_PATH_X])
    AC_REQUIRE([AC_PATH_XTRA])

    dnl save the environment
    AC_LANG_SAVE
    AC_LANG_C
    pg_x11_x_libs="$LIBS"
    pg_x11_cflags="$CFLAGS"

    AC_MSG_CHECKING(for X11 libraries)

    dnl See if we can link with X11 as is.
    LIBS="-lX11 $X_PRE_LIBS $X_LIBS $X_EXTRA_LIBS"
    CFLAGS="$CFLAGS $X_CFLAGS"
    AC_TRY_LINK([char XBell();],
      [XBell();],
      pg_cv_libs_x11="-lX11",
      pg_cv_libs_x11=no)

    dnl do we need -ldl to go with the -lX11?
    if test "x$pg_cv_libs_x11" = xno; then
      LIBS="-lX11 -ldl $X_PRE_LIBS $X_LIBS $X_EXTRA_LIBS"
      AC_TRY_LINK([char XBell();],
        [XBell();],
        pg_cv_libs_x11="-lX11 -ldl",
        pg_cv_libs_x11=no)
    fi

    dnl OpenGL, GLX and Motif libraries sometimes require libXext so lets 
    dnl include it in the linker's command line just in case.  This way we
    dnl do not need to check it all the time.
    if test "x$pg_cv_libs_x11" = xno; then
      :
    else
      LIBS="-lXext $pg_cv_libs_x11 $X_PRE_LIBS $X_LIBS $X_EXTRA_LIBS $pg_x11_x_libs"
      AC_TRY_LINK([char XextAddDisplay();],
        [XextAddDisplay();],
        pg_cv_libs_x11="-lXext $pg_cv_libs_x11",
        pg_cv_libs_x11=no)
    fi

    dnl At least OpenGL libraries on SGI and Linux sometimes require libXmu 
    dnl (Linux when using static libraries) so add -lXmu to LIBS.  Furthermore,
    dnl on Linux libXmu seems to require -lXt which on the other hand depends
    dnl on libraries -lSM and -lICE but luckily these are supplied by 
    dnl $X_PRE_LIBS.
    if test "x$pg_cv_libs_x11" = xno; then
      :
    else
      dnl try to link with -lXt
      LIBS="-lXt $pg_cv_libs_x11 $X_PRE_LIBS $X_LIBS $X_EXTRA_LIBS $pg_x11_x_libs"
      AC_TRY_LINK([char XtOpenDisplay();],
        [XtOpenDisplay();],
        pg_libs_xt="-lXt",
        pg_libs_xt=no)
  
      dnl try to link with -lXmu now that we have -lXt
      if test "x$pg_libs_xt" = xno; then
        :
      else
        LIBS="-lXmu $pg_libs_xt $pg_cv_libs_x11 $X_PRE_LIBS $X_LIBS $X_EXTRA_LIBS $pg_x11_x_libs"
        AC_TRY_LINK([char XmuNameOfAtom();],
          [XmuNameOfAtom();],
          pg_libs_xmu="-lXmu",
          pg_libs_xmu=no)
      fi
  
      if test "x$pg_libs_xt" != xno; then
        pg_cv_libs_x11="$pg_libs_xt $pg_cv_libs_x11"
      fi
      if test "x$pg_libs_xmu" != xno; then
        pg_cv_libs_x11="$pg_libs_xmu $pg_cv_libs_x11"
      fi
    fi

    AC_MSG_RESULT($pg_cv_libs_x11)

    dnl restore environment
    LIBS="$pg_x11_x_libs"
    CFLAGS="$pg_x11_cflags"
    AC_LANG_RESTORE

    dnl define X11 if the X11 library was found
    if test "x$pg_cv_libs_x11" = xno; then
      :
    else
      CFLAGS="$CFLAGS $X_CFLAGS"
      CXXFLAGS="$CXXFLAGS $X_CFLAGS"
      CPPFLAGS="$CPPFLAGS -DX11"
      GFXLIBS="$pg_cv_libs_x11 $X_PRE_LIBS $X_LIBS $X_EXTRA_LIBS $GFXLIBS"
    fi

    dnl save the environment
    AC_LANG_SAVE
    AC_LANG_C
    pg_x11_x_libs="$LIBS"
    pg_x11_cflags="$CFLAGS"

    dnl check for XF86 video mode extensions
    AC_MSG_CHECKING(for XFree86 video mode extension)
    if test "x$pg_cv_libs_x11" = xno; then
      :
    else
      LIBS="-lXxf86vm $GFXLIBS"
      AC_TRY_LINK([char XF86VidModeGetViewPort();],
        [XF86VidModeGetViewPort();],
        pg_cv_xf86=yes,
        pg_cv_xf86=no)
    fi

    AC_MSG_RESULT($pg_cv_xf86)

    dnl restore environment
    LIBS="$pg_x11_x_libs"
    CFLAGS="$pg_x11_cflags"
    AC_LANG_RESTORE

    dnl export result
    if test x"$pg_cv_xf86" = x"yes"; then
      CPPFLAGS="$CPPFLAGS -DXF86VMODE"
      GFXLIBS="-lXxf86vm $GFXLIBS"
    fi

    dnl export libs to Makefile
    AC_SUBST(GFXLIBS)
  fi

  dnl define conditionals for branching in Makefiles
  AM_CONDITIONAL(X11, test "x$pg_cv_libs_x11" != xno)
])


dnl
dnl Check for X11 OpenGL libraries (GLX).
dnl
dnl Cached variables:
dnl   pg_cv_libs_opengl
dnl   pg_cv_libs_glx
dnl
dnl Compier defines affected:
dnl   OPENGL
dnl   GLX
dnl
dnl Automake variables affected:
dnl   @GFXLIBS@
dnl
AC_DEFUN([LIBGLX],[
  AC_REQUIRE([LIBX11])
  pg_cv_libs_glx=no
  pg_cv_libs_opengl=no

  dnl Prefer mesage
  AC_ARG_WITH(mesa,
    [  --with-mesa             prefer Mesa over native OpenGL],
    [prefer_mesa="$withval"],
    [prefer_mesa="no"])

  dnl only continue with glx if x11 was found
  if test "x$pg_cv_libs_x11" != xno; then

    dnl save enviroment
    AC_LANG_SAVE
    AC_LANG_C
    pg_opengl_libs="$LIBS"

    dnl check for OpenGL
    AC_MSG_CHECKING(for OpenGL)
    LIBS="-lGL -lGLU -lm $XLIBS $pg_opengl_libs"

    dnl prefer native OpenGL libraries unless prohibited by --with-mesa
    if test x"$prefer_mesa" = x"no"; then
      AC_TRY_LINK([char glLoadIdentity ();],
        [glLoadIdentity();],
        pg_cv_libs_opengl="-lGL -lGLU -lm",
        pg_cv_libs_opengl=no)

      dnl OpenGL may depend on pthreads so lets try to add necessary options
      dnl to linker line.  In FreeBSD we must use -pthread instead of 
      dnl -lpthread.
      if test "x$pg_cv_libs_opengl" = xno; then
        LIBS="-lGL -lGLU -lm -pthread $XLIBS $pg_opengl_libs"
        AC_TRY_LINK([char glLoadIdentity ();],
          [glLoadIdentity();],
          pg_cv_libs_opengl="-lGL -lGLU -lm -pthread",
          pg_cv_libs_opengl=no)
      fi
      if test "x$pg_cv_libs_opengl" = xno; then
        LIBS="-lGL -lGLU -lm -lpthread $XLIBS $pg_opengl_libs"
        AC_TRY_LINK([char glLoadIdentity ();],
          [glLoadIdentity();],
          pg_cv_libs_opengl="-lGL -lGLU -lm -lpthread",
          pg_cv_libs_opengl=no)
      fi
    fi

    dnl check for Mesa.  At first check if there are debugging versions
    dnl available on Mesa libraries and use them if the user specified
    dnl --enable-debug switch in the command line.  Mesa debugging
    dnl libraries are not typically present and their names might
    dnl be different from those expected.
    if test "x$pg_cv_libs_opengl" = xno -a "x$pg_cv_debug" = xtrue; then
      LIBS="-lMesaGLD -lMesaGLUD -lm $XLIBS $pg_opengl_libs"
      AC_TRY_LINK([#include <GL/osmesa.h>],
        [glLoadIdentity();],
        pg_cv_libs_opengl="-lMesaGLD -lMesaGLUD -lm",
        pg_cv_libs_opengl=no)
    fi

    dnl check for optimized Mesa libraries
    if test "x$pg_cv_libs_opengl" = xno; then
      LIBS="-lMesaGL -lMesaGLU -lm $XLIBS $pg_opengl_libs"
      AC_TRY_LINK([#include <GL/osmesa.h>],
        [glLoadIdentity();],
        pg_cv_libs_opengl="-lMesaGL -lMesaGLU -lm",
        pg_cv_libs_opengl=no)
    fi

    AC_MSG_RESULT($pg_cv_libs_opengl)

    dnl restore compiler flags
    LIBS="$pg_opengl_libs"
    AC_LANG_RESTORE

    dnl set compiler flags to suit selected libraries
    if test "x$pg_cv_libs_opengl" = xno; then
      :
    else
      CPPFLAGS="$CPPFLAGS -DOPENGL"
      GFXLIBS="$pg_cv_libs_opengl $GFXLIBS"
    fi

    dnl check for glx
    if test "x$pg_cv_libs_opengl" != xno; then
      AC_MSG_CHECKING(for glx)

      dnl save environment
      AC_LANG_SAVE
      AC_LANG_C
      pg_glx_libs="$LIBS"

      dnl Try to link with default libraries to see if we need anything
      dnl special at all.  We do not need to check -lXext here since macro
      dnl LIBX11 already makes sure that it is included in $XLIBS.
      LIBS="$LIBS $GFXLIBS $XLIBS"
      AC_TRY_LINK(
        [#include <GL/glx.h>],
        [(void)glXChooseVisual(0L,0,0L);],
        [pg_cv_libs_glx=""],
        [pg_cv_libs_glx=no])
  
      dnl tell user about the result we reached
      if test x"$pg_cv_libs_glx" = x; then
        AC_MSG_RESULT(yes)
      else
        AC_MSG_RESULT($pg_cv_libs_glx)
      fi
  
      dnl restore compiler flags
      LIBS="$pg_glx_libs"
      AC_LANG_RESTORE

      dnl define GLX if the glx library was found
      if test "x$pg_cv_libs_glx" = xno; then
        :
      else
        CPPFLAGS="$CPPFLAGS -DGLX"
        if test "x$pg_cv_libs_glx" = "x"; then
          :
        else
          GFXLIBS="$pg_cv_libs_glx $GFXLIBS"
        fi
      fi
    fi

    dnl export libs Makefile
    AC_SUBST(GFXLIBS)
  else
    AC_MSG_CHECKING(for GLX)
    AC_MSG_RESULT(missing X)
  fi

  dnl define conditionals for branching in Makefiles
  AM_CONDITIONAL(GLX, test "x$pg_cv_libs_glx" != xno)
])


dnl
dnl Check for OpenGL Utility Toolkit (glut).
dnl
dnl Cached variables:
dnl   pg_cv_headers_glut
dnl   pg_cv_libs_glut
dnl
dnl Compiler defines affected:
dnl   GLUT
dnl
dnl Automake variables affected:
dnl   @GFXLIBS@
dnl
AC_DEFUN([LIBGLUT],[
  AC_REQUIRE([LIBGLX])

  dnl clear those not really cached variables
  pg_cv_headers_glut=no
  pg_cv_libs_glut=no

  dnl let user specify a non-conventional path for libglut
  AC_ARG_WITH(glut,
  [  --with-glut[[=DIR]]       specify location of glut headers and libraries],
  [
    GLUTDIR="$withval"
  ],[
    GLUTDIR=""
  ])

  dnl check for libglut if OpenGL or Mesa libraries exist
  if test "x$pg_cv_libs_opengl" != "xno"; then
    dnl save compiler flags
    AC_LANG_SAVE
    AC_LANG_C
    pg_glut_libs="$LIBS"
    pg_glut_cflags="$CFLAGS"

    AC_MSG_CHECKING(for GLUT headers)

    dnl look for glut.h
    pg_cv_headers_glut=no
    for dir in \
        "${GLUTDIR}" \
        "${GLUTDIR}/include" \
        "${prefix}/include" \
        "/usr/include" \
        "/usr/local/include" \
        "/opt/graphics/OpenGL" 
    do
      if test -r "$dir/GL/glut.h"  -o  "x$dir" = x; then
        dnl compile small program to make sure that everything is OK
        if test "x$dir" != x; then
          CFLAGS="$pg_glut_cflags -I${dir}"
        else
          CFLAGS="$pg_glut_cflags"
        fi
        AC_TRY_COMPILE([#include <GL/glut.h>],
          [glutSolidTeapot(1);],
          pg_cv_headers_glut="$dir")
        if test "x$pg_cv_headers_glut" != xno; then
          break
        fi
      fi
    done
   
    dnl output results to the user
    if test "x$pg_cv_headers_glut" != x; then
      pg_cv_headers_glut_result="$pg_cv_headers_glut"
    else
      pg_cv_headers_glut_result="in default path"
    fi
    AC_MSG_RESULT($pg_cv_headers_glut_result)

    dnl look for glut library
    pg_cv_libs_glut=no
    if test "x$pg_cv_headers_glut" != xno; then
      dnl prepare list of possible library names
      if test "x$pg_cv_debug" = xtrue; then
        glutlibslist="glutD glut"
      else
        glutlibslist="glut"
      fi

      dnl look for glut library from directories below
      AC_MSG_CHECKING(for GLUT library)
      for dir in \
          "${GLUTDIR}" \
          "${GLUTDIR}/lib32" \
          "${GLUTDIR}/lib" \
          "${prefix}/lib" \
          "/usr/lib32" \
          "/usr/lib" 
      do
        dnl make compiler switch that takes directory in consideration
        if test "x$dir" != x; then
          dirswitch="-L$dir"
        else
          dirswitch=""
        fi
  
        dnl check for each name under each directory
        for lib in $glutlibslist; do
          LIBS="$dirswitch -l$lib $GFXLIBS $XLIBS $pg_glut_libs "
 
          dnl try to link with a simple program
          AC_TRY_LINK([char glutSolidTeapot();],
            [glutSolidTeapot();],
            pg_cv_libs_glut="$dirswitch -l$lib")
          if test "x$pg_cv_libs_glut" != xno; then
            dnl succeeded in linking
            break
          fi
        done
        if test "x$pg_cv_libs_glut" != xno; then
          break
        fi
      done

      dnl output results to the user
      AC_MSG_RESULT($pg_cv_libs_glut)
    fi

    dnl restore compiler flags
    LIBS="$pg_glut_libs"
    CFLAGS="$pg_glut_cflags"
    AC_LANG_RESTORE
  else
    dnl no OpenGL
    AC_MSG_CHECKING(for GLUT)
    AC_MSG_RESULT(missing GLX)
    pg_cv_libs_glut=no
  fi

  dnl set compiler flags to suit selected libraries
  if test "x$pg_cv_libs_glut" != xno; then
    dnl enable GLUT and add include directory to the command line
    CPPFLAGS="$CPPFLAGS -DGLUT"
    if test "x$pg_cv_headers_glut" = x -o "x$pg_cv_headers_glut" = x.; then
      :
    else
      CPPFLAGS="$CPPFLAGS -I${pg_cv_headers_glut}"
    fi

    dnl add libglut to OpenGL libraries
    GFXLIBS="$pg_cv_libs_glut $GFXLIBS"
    AC_SUBST(GFXLIBS)
  fi

  dnl define conditionals for branching in Makefiles
  AM_CONDITIONAL(GLUT, test "x$pg_cv_libs_glut" != xno)
])
