/*
 * basename.c - get trailing component of path
 * 
 * Copyright (C) 2005  Toni Ronkko
 * 
 * Peach Gardens is free software; you can redistribute it and/or modify 
 * it under the terms of the GNU General Public License as published by the
 * Free Software Foundation; either version 2 of the License, or (at your
 * option) any later version.
 *
 * Peach Gardens is distributed in the hope that it will be useful, but 
 * WITHOUT ANY WARRANTY; without even the implied warranty of 
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU 
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with Peach Gardens; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 * 
 * 
 * Dec 23 2004, Toni Ronkko <toni at users.playstation2-linux.com>
 *
 * $Id: basename.c,v 1.1 2005/01/09 21:10:24 tronkko Exp $
 *
 * $Log: basename.c,v $
 * Revision 1.1  2005/01/09 21:10:24  tronkko
 * Initial revision
 *
 *
 */
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <assert.h>

#include "basename.h"


/*
 * Get trailing component of file name fn.  For example, given file name
 * "/tmp/file.tmp", return "file.tmp".  Likewise, given "/usr/local/",
 * return "local".  Note that the function may modify the input parameter in
 * some cases.  Always copy the input parameter if you need it.
 */
char*
my_basename(
    char *fn)
{
  char *p;
  assert (fn != NULL);
  assert (fn[0] != '\0');

  /* find trailing slash */
  p = strrchr (fn, '/');
  if (p != NULL) {
    /* there is a slash in the file name */
    if (p[1] != '\0') {
      /* there is a file name following the last slash */
      ++p;
    } else {
      /* no file name following slash; strip trailing '/' characters  */
      while (fn < p  &&  p[-1] == '/') --p;
      *p = '\0';
      
      /* look for the slash preceeding the trailing slashes */
      while (fn < p  &&  p[-1] != '/') --p;
    }
  } else {
    /* there is no path in the file name */
    p = fn;
  }
  return p;
}
