/*
 * birds.h - birds flying all over screen
 * 
 * Copyright (C) 2005  Toni Ronkko
 * 
 * Peach Gardens is free software; you can redistribute it and/or modify 
 * it under the terms of the GNU General Public License as published by the
 * Free Software Foundation; either version 2 of the License, or (at your
 * option) any later version.
 *
 * Peach Gardens is distributed in the hope that it will be useful, but 
 * WITHOUT ANY WARRANTY; without even the implied warranty of 
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU 
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with Peach Gardens; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * 
 * Dec 6 2004, Toni Ronkko <toni at users.playstation2-linux.com>
 *
 * $Id: birds.h,v 1.2 2005/04/03 16:20:01 tronkko Exp $
 *
 * $Log: birds.h,v $
 * Revision 1.2  2005/04/03 16:20:01  tronkko
 * birds move relative to view
 *
 * Revision 1.1  2005/01/09 21:05:12  tronkko
 * Initial revision
 *
 *
 */
#ifndef BIRDS_H
#define BIRDS_H


/* how often birds flap their wings (100 is once per 100 ticks) */
#define BIRD_FLAP_INTERVAL 70

/* maximum number of birds in the air at the same time */
#define MAX_BIRDS 3

/* default frequency of birds appearing (20 means every frame) */
#define BIRD_GENERATION 3000

/* default frequency of birds changing flying direction */
#define BIRD_DIRECTION 1000

/* scale of birds on screen (2 is twice the original) */
#define BIRD_SCALE 2


struct view; /*forward-decl*/

/* paint active birds to screen */
void draw_birds (struct view *vp);

/* move and animate birds */
void animate_birds (struct view *vp);

/* generate new birds to the sky */
void generate_birds (struct view *vp);

/* insert new bird at specific location with direction and speed */
int insert_bird(
    struct view *vp, float x, float y, float dx, float dy, float s);

/* set odds of birds generated and birds appearing from another direction */
void bird_frequency (long int generation, long int change);
  
/* remove all birds */
void reset_birds (void);

/* initialize/deinitialize birds */
void init_birds (void);
void done_birds (void);

#endif /*BIRDS_H*/
