/*
 * chipset.h - manage chipsets
 * 
 * Copyright (C) 2005  Toni Ronkko
 * 
 * Peach Gardens is free software; you can redistribute it and/or modify 
 * it under the terms of the GNU General Public License as published by the
 * Free Software Foundation; either version 2 of the License, or (at your
 * option) any later version.
 *
 * Peach Gardens is distributed in the hope that it will be useful, but 
 * WITHOUT ANY WARRANTY; without even the implied warranty of 
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU 
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with Peach Gardens; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 * 
 * 
 * Dec 23 2004, Toni Ronkko <toni at users.playstation2-linux.com>
 *
 * $Id: chipset.h,v 1.2 2005/01/09 21:05:39 tronkko Exp $
 *
 * $Log: chipset.h,v $
 * Revision 1.2  2005/01/09 21:05:39  tronkko
 * simplified resource management
 *
 * Revision 1.1  2004/12/29 00:08:10  tronkko
 * Initial revision
 *
 *
 */
#ifndef CHIPSET_H
#define CHIPSET_H

#include "image.h"

struct chip; /*forward-decl*/


/*
 * Information on a single chipset.  In Peach Gardens, chipset is a
 * container of chip objects.  The main purpose of the chipset entity is to
 * provide storage and retrieval functions for chips.
 */
struct chipset
{
  int chipw, chiph;                            /* width & height of chip */
  int nchips;                                  /* # of chips in set */
  struct chip *chips;                          /* the chips */
  struct image img;                            /* image file */
};


/* allocation of chipsets */
void init_chipset (struct chipset *chp, int n, int w,int h, const char *fn);
void done_chipset (struct chipset *chp);

/* find chip within chipset */
#define get_chip_macro(chp,i) (&(chp)->chips[(i)])
struct chip *get_chip_function (struct chipset *chp, int i);

/* use macro version for speed */
#ifdef NDEBUG
# define get_chip(chp,i) get_chip_macro(chp,i)
#else /*debug*/
# define get_chip(chp,i) get_chip_function(chp,i)
#endif /*debug*/

#endif /*CHIPSET_H*/
