/*
 * debug.h - utilities for debugging programs
 * 
 * Copyright (C) 2005  Toni Ronkko
 * 
 * Peach Gardens is free software; you can redistribute it and/or modify 
 * it under the terms of the GNU General Public License as published by the
 * Free Software Foundation; either version 2 of the License, or (at your
 * option) any later version.
 *
 * Peach Gardens is distributed in the hope that it will be useful, but 
 * WITHOUT ANY WARRANTY; without even the implied warranty of 
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU 
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with Peach Gardens; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 * 
 * 
 * Nov  1 2004, Toni Ronkko <toni at users.playstation2-linux.com>
 *
 * $Id: debug.c,v 1.3 2005/06/22 13:09:32 tronkko Exp $
 *
 * $Log: debug.c,v $
 * Revision 1.3  2005/06/22 13:09:32  tronkko
 * print error message to graphics dialog
 *
 * Revision 1.2  2005/04/03 16:29:43  tronkko
 * abort_msg is a function
 *
 * Revision 1.1  2005/01/09 21:05:48  tronkko
 * Initial revision
 *
 *
 */
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <assert.h>
#include <stdarg.h>

#include "config.h"
#include "graphics.h"
#include "debug.h"


/* maximum length of error message */
#define MAX_MSG 1024


/*
 * Print data bytes to stdout for debugging.  This is useful for debugging
 * DMA packets, for example.
 */
void
dump_bytes(
    const void *data, int len)
{
  int i;
  const int rowlen = 16;
  const unsigned char *p = (const unsigned char*) data;
  const unsigned char *e = ((const unsigned char*) data) + len;  

  /* print data in rows */
  while (p < e) {
    printf ("%8p: ", p);

    /* print single row */
    i = 0;
    while (i < rowlen  &&  p < e) {
      printf (" %02x", *p++);
      i++;
    }
    
    printf ("\n");
  }  
}


/*
 * Terminate application with a formatted message.  In Nokia 770 Internet
 * Tabler regular users have no access to console window, and the message
 * will be shown in a graphics window.
 */
void
abort_msg(
    const char *fmt, ...)
{
  char msg[MAX_MSG];

  va_list ap;
  va_start(ap, fmt);

  /* format error message */
  strcpy (msg, "Fatal error: ");
  vsprintf (strchr(msg, '\0'), fmt, ap);
  assert (strlen (msg) < MAX_MSG);
  msg[MAX_MSG - 1] = '\0';
    
  /* show error dialog */
  error (msg);
  
  va_end(ap);
  abort ();
}
