/*
 * debug.h - utilities for debugging programs
 * 
 * Copyright (C) 2005  Toni Ronkko
 * 
 * Peach Gardens is free software; you can redistribute it and/or modify 
 * it under the terms of the GNU General Public License as published by the
 * Free Software Foundation; either version 2 of the License, or (at your
 * option) any later version.
 *
 * Peach Gardens is distributed in the hope that it will be useful, but 
 * WITHOUT ANY WARRANTY; without even the implied warranty of 
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU 
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with Peach Gardens; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 * 
 * 
 * Nov  1 2004, Toni Ronkko <toni at users.playstation2-linux.com>
 *
 * $Id: debug.h,v 1.3 2005/04/03 16:21:37 tronkko Exp $
 *
 * $Log: debug.h,v $
 * Revision 1.3  2005/04/03 16:21:37  tronkko
 * abort_msg is a function, check args with gcc attribute
 *
 * Revision 1.2  2005/01/09 21:11:19  tronkko
 * abort_msg and error reporting macros
 *
 * Revision 1.1  2004/11/20 10:56:59  tronkko
 * Initial revision
 *
 *
 */
#ifndef DEBUG_H
#define DEBUG_H

#include "feat.h"

/* dump data bytes to stdout */
void dump_bytes (const void *data, int len);

/* abort with a message */
#ifdef __GNUC__
void abort_msg (const char *fmt, ...)
  __attribute__ ((noreturn)) __attribute__ ((format(printf,1,2)));
#else
void abort_msg (const char *fmt, ...);
#endif

/* shorthands for common errors */
#define out_of_memory()   abort_msg("out of memory")
#define invalid_escape()  abort_msg("invalid escape sequence")

#endif

/*
 * Print extra debugging info.  Note that the code below has been
 * intentionally placed outside any file level #ifdef .. #endif blocks to 
 * enable and disable debugging for parts of a single file.
 */
#undef DEBUG
#if ENABLE_DEBUG
# define DEBUG(x) \
  do{ \
    printf("%s:%d:  ", __FILE__, __LINE__); \
    printf x ; \
    printf("\n"); \
    fflush(stdout); \
  }while(0)
#else
# define DEBUG(x) ((void)0)
#endif
