/*
 * font10x16.c - 10x16 fixed width font
 * 
 * Copyright (C) 2005  Toni Ronkko
 * 
 * Peach Gardens is free software; you can redistribute it and/or modify 
 * it under the terms of the GNU General Public License as published by the
 * Free Software Foundation; either version 2 of the License, or (at your
 * option) any later version.
 *
 * Peach Gardens is distributed in the hope that it will be useful, but 
 * WITHOUT ANY WARRANTY; without even the implied warranty of 
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU 
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with Peach Gardens; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * 
 * Apr  3 2005, Toni Ronkko <toni at users.playstation2-linux.com>
 *
 * $Id: font10x16.c,v 1.1 2005/06/22 13:10:13 tronkko Exp $
 *
 * $Log: font10x16.c,v $
 * Revision 1.1  2005/06/22 13:10:13  tronkko
 * Initial revision
 *
 *
 */
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <assert.h>

#include "config.h"
#include "font10x16.h"
#include "font.h"
#include "locate.h"
#include "text.h"
#include "graphics.h"

#ifdef MAEMO
# include <gdk/gdk.h>
# include "maemo.h"
#endif


/* replacement font for Nokia 770 Internet Tablet */
#define MAEMO_FONT "-*-courier-medium-r-normal-*-16-*-*-*-*-*-iso8859-*"


/* get information on single glyph */
#ifdef MAEMO
static const struct glyph *info (int c);
#endif

/* draw glyph to current text pos */
#ifdef MAEMO
static int draw (int c);
#endif


/* publicly usable font structure */
struct font *font10x16 = NULL;
static struct font _font10x16;

/*
 * Character widths etc for 10x16 font.    Note that the measurements in
 * this array are not very precise.  The font started its life as a fixed
 * width font, and most of the metrics do not really describe the
 * individual glyphs very well.
 */
#ifndef MAEMO
static const struct glyph glyphs[] = {
/*  code   x   y   w  h,  spc asc desc*/
  { ' ',   0,  0, 10, 16, 0,0, 14,2 },
  { '!',  10,  0, 10, 16, 0,0, 14,2 },
  { '"',  20,  0, 10, 16, 0,0, 14,2 },
  { '#',  30,  0, 10, 16, 0,0, 14,2 },
  { '$',  40,  0, 10, 16, 0,0, 14,2 },
  { '%',  50,  0, 10, 16, 0,0, 14,2 },
  { '\'', 70,  0, 10, 16, 0,0, 14,2 },
  { '(',  80,  0, 10, 16, 0,0, 14,2 },
  { ')',  90,  0, 10, 16, 0,0, 14,2 },
  { '*', 100,  0, 10, 16, 0,0, 14,2 },
  { '+', 110,  0, 10, 16, 0,0, 14,2 },
  
  { ',',   0, 16, 10, 16, 0,0, 14,2 },
  { '-',  10, 16, 10, 16, 0,0, 14,2 },
  { '.',  23, 27,  4,  4, 2,2,  3,2 },
  { '/',  30, 16,  9, 16, 0,0, 14,2 },
  { '0',  40, 16, 10, 16, 0,0, 14,2 },
  { '1',  52, 16,  6, 14, 1,2, 14,2 },
  { '2',  60, 16, 10, 16, 0,0, 14,2 },
  { '3',  70, 16, 10, 16, 0,0, 14,2 },
  { '4',  80, 16, 10, 16, 0,0, 14,2 },
  { '5',  90, 16, 10, 16, 0,0, 14,2 },
  { '6', 100, 16, 10, 16, 0,0, 14,2 },
  { '7', 110, 16, 10, 16, 0,0, 14,2 },
  
  { '8',   0, 32, 10, 16, 0,0, 14,2 },
  { '9',  10, 32, 10, 16, 0,0, 14,2 },
  { ':',  20, 32, 10, 16, 0,0, 14,2 },
  { ';',  30, 32, 10, 16, 0,0, 14,2 },
  { '<',  40, 32, 10, 16, 0,0, 14,2 },
  { '=',  50, 32, 10, 16, 0,0, 14,2 },
  { '>',  60, 32, 10, 16, 0,0, 14,2 },
  { '?',  70, 32, 10, 16, 0,0, 14,2 },
  { '@',  81, 32,  8, 16, 1,2, 14,2 },
  { 'A',  90, 32, 10, 16, 0,0, 14,2 },
  { 'B', 100, 32, 10, 16, 0,0, 14,2 },
  { 'C', 110, 32, 10, 16, 0,0, 14,2 },
  
  { 'D',   0, 48, 10, 16, 0,0, 14,2 },
  { 'E',  10, 48, 10, 16, 0,0, 14,2 },
  { 'F',  20, 48,  9, 16, 0,0, 14,2 },
  { 'G',  30, 48, 10, 16, 0,0, 14,2 },
  { 'H',  40, 48, 10, 16, 0,0, 14,2 },
  { 'I',  52, 48,  6, 16, 1,2, 14,2 },
  { 'J',  60, 48, 10, 16, 0,0, 14,2 },
  { 'K',  70, 48, 10, 16, 0,0, 14,2 },
  { 'L',  80, 48, 10, 16, 0,0, 14,2 },
  { 'M',  90, 48, 10, 16, 0,0, 14,2 },
  { 'N', 100, 48, 10, 16, 0,0, 14,2 },
  { 'O', 110, 48, 10, 16, 0,0, 14,2 },
   
  { 'P',   0, 64,  9, 16, 0,0, 14,2 },
  { 'Q',  10, 64, 10, 16, 0,0, 14,2 },
  { 'R',  20, 64, 10, 16, 0,0, 14,2 },
  { 'S',  30, 64, 10, 16, 0,0, 14,2 },
  { 'T',  40, 64, 10, 16, 0,0, 14,2 },
  { 'U',  50, 64, 10, 16, 0,0, 14,2 },
  { 'V',  60, 64, 10, 16, 0,0, 14,2 },
  { 'W',  71, 64,  8, 16, 1,2, 14,2 },
  { 'X',  80, 64, 10, 16, 0,0, 14,2 },
  { 'Y',  90, 64, 10, 16, 0,0, 14,2 },
  { 'Z', 100, 64, 10, 16, 0,0, 14,2 },
  { '[', 110, 64, 10, 16, 0,0, 14,2 },
  
  { '\\',  0, 80, 10, 16, 0,0, 14,2 },
  { '}',  10, 80, 10, 16, 0,0, 14,2 },
  { '^',  20, 80, 10, 16, 0,0, 14,2 },
  { '`',  40, 80, 10, 16, 0,0, 14,2 },
  { 'a',  50, 80, 10, 14, 0,0, 14,2 },
  { 'b',  60, 80, 10, 14, 0,0, 14,2 },
  { 'c',  70, 80, 10, 14, 0,0, 14,2 },
  { 'd',  80, 80, 10, 14, 0,0, 14,2 },
  { 'e',  90, 80, 10, 14, 0,0, 14,2 },
  { 'f', 100, 80, 10, 14, 0,0, 14,2 },
  { 'g', 110, 80, 10, 16, 0,0, 14,2 },
  
  { 'h',   0, 96, 10, 14, 0,0, 14,2 },
  { 'i',  12, 96,  6, 14, 1,1, 14,2 },
  { 'j',  20, 96, 10, 16, 0,0, 14,2 },
  { 'k',  30, 96, 10, 14, 0,0, 14,2 },
  { 'l',  42, 96,  6, 14, 1,1, 14,2 },
  { 'm',  51, 96,  8, 14, 1,2, 14,2 },
  { 'n',  60, 96, 10, 14, 0,0, 14,2 },
  { 'o',  70, 96, 10, 14, 0,0, 14,2 },
  { 'p',  80, 96, 10, 16, 0,0, 14,2 },
  { 'q',  90, 96, 10, 16, 0,0, 14,2 },
  { 'r', 100, 96, 10, 14, 0,0, 14,2 },
  { 's', 110, 96, 10, 14, 0,0, 14,2 },
  
  { 't',   0,112, 10, 14, 0,0, 14,2 },
  { 'u',  11,116,  8, 10, 1,1, 10,2 },
  { 'v',  20,112, 10, 16, 0,0, 14,2 },
  { 'w',  31,112,  8, 16, 1,2, 14,2 },
  { 'x',  40,112, 10, 14, 0,0, 14,2 },
  { 'y',  50,112, 10, 16, 0,0, 14,2 },
  { 'z',  61,116,  7, 10, 1,2, 10,2 },

  { '\0',  0,  0,  0,  0, 0,0,  0,0 } 
};
#endif

/* has module been initialized? */
static int initialized = 0;

/* gdk font that best matches 10x16 bitmap font */
#ifdef MAEMO
static GdkFont *gdkfont;
#endif


/*
 * Get glyph for character c.
 *
 * Note that in Nokia 770 Internet Tablet we cannot really initialize fonts
 * until gtk_init() has been called.  However, that function cannot be
 * called at all from a non-graphical application, so we cannot load fonts
 * either.  Hence, fonts must be loaded here when a font is being used for
 * the very first time.
 */
#ifdef MAEMO
static const struct glyph*
info(
    int c)
{
  assert (initialized);
  assert (0 <= c  &&  c < MAX_GLYPHS);

  if (gdkfont == NULL) {
    /* load X font */
    gdkfont = gdk_font_load (MAEMO_FONT);
    assert (gdkfont != NULL);
    
    /* initialize glyphs table */
    maemo_init_glyphs (gdkfont, _font10x16.glyphs, MAX_GLYPHS);
  }
  return &_font10x16.glyphs[c];
}
#endif /*!MAEMO*/


/*
 * Draw glyph c to current text pos.  Returns character width.
 */
#ifdef MAEMO
static int 
draw(
    int c)
{
  int w = 0;                                   /* character width */
  const struct glyph *gp;                      /* glyph data */

  assert (initialized);
  assert (font10x16 == &_font10x16);
  
  gp = info (c);
  if (gp != NULL) {
    /* paint character to screen */
    if (c != ' ') {
      maemo_draw_glyph (gdkfont, textx(), texty(), c);
    }
    
    /* compute width of character */
    w = gp->lead + gp->w + gp->follow;
  }

  return w;
}
#endif /*!MAEMO*/


/*
 * Load 10x16 font to memory.
 */
void
init_font10x16 (void)
{
  assert (!initialized);

#ifdef MAEMO

  /* initialize font structure */
  init_font (&_font10x16);
  font10x16 = &_font10x16;
  font10x16->info = info;
  font10x16->draw = draw;

#else /*!MAEMO*/

  /* set up font */
  init_bitmap_font (&_font10x16, locate("10x16.pcx"), glyphs);
  font10x16 = &_font10x16;

#endif /*!MAEMO*/

  initialized = 1;
}


/*
 * Unload font from memory.
 */
void
done_font10x16 (void)
{
  assert (initialized);

#ifdef MAEMO

  /* release font */
  if (gdkfont != NULL) {
    gdk_font_unref(gdkfont);
    gdkfont = NULL;
  }

#endif

  /* de-initialize font structure */
  assert (font10x16 == &_font10x16);
  done_font (&_font10x16);
  font10x16 = NULL;

  initialized = 0;
}
