/*
 * graphics.h - graphics primitives
 * 
 * Copyright (C) 2005  Toni Ronkko
 * 
 * Peach Gardens is free software; you can redistribute it and/or modify 
 * it under the terms of the GNU General Public License as published by the
 * Free Software Foundation; either version 2 of the License, or (at your
 * option) any later version.
 *
 * Peach Gardens is distributed in the hope that it will be useful, but 
 * WITHOUT ANY WARRANTY; without even the implied warranty of 
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU 
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with Peach Gardens; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 * 
 * 
 * Jan  3 2004, Toni Ronkko <toni at users.playstation2-linux.com>
 *
 * $Id: graphics.h,v 1.4 2005/06/22 13:10:24 tronkko Exp $
 *
 * $Log: graphics.h,v $
 * Revision 1.4  2005/06/22 13:10:24  tronkko
 * print error dialog
 *
 * Revision 1.3  2005/04/03 16:30:19  tronkko
 * flush_gfx(), fade()
 *
 * Revision 1.2  2005/01/09 21:11:50  tronkko
 * blend was renamed to textblt
 *
 * Revision 1.1  2004/11/20 10:59:16  tronkko
 * Initial revision
 *
 */
#ifndef GRAPHICS_H
#define GRAPHICS_H

struct image; /* forward-decl */

/* initialize graphics libraries (does not open graphics window) */
void init_graphics (void);

/* de-initialize graphics libraries returning to text mode */
void done_graphics (void);

/* open graphics window */
int open_screen (void);

/* close graphics window returning to text mode */
void close_screen (void);

/* clear graphics window resetting any drawing modes */
void clear (void);

/* set background color used by clear() */
void setbg (int r, int g, int b);

/* set foregroung color */
void setfg (int r, int g, int b);

/* draw line (x1,y1)-(x2,y2) using current pen */
void line (int x1, int y1, int x2, int y2);

/* draw triangle (x1,y1)-(x2,y2),-(x3,y3) using current pen */
void triangle (int x1, int y1, int x2, int y2, int x3, int y3);

/* draw image */
void bitblt(
    struct image *imp,
    int sx1, int sy1, int sx2, int sy2,
    int dx1, int dy1, int dx2, int dy2);

/* draw image with blending operations (used for fonts) */
void textblt(
    struct image *imp,
    int sx, int sy, int w, int h,
    int dx, int dy);
  
/* swap double buffers */
void swap_buffers (void);

/* get width and height of graphics screen */
int screenwidth (void);
int screenheight (void);

/* flush drawing queues and return immediately */
void flush_gfx (void);

/* fade to foreground color */
void fade (int x1, int y1, int x2, int y2, int alpha);

/* print error message to graphics window */
void error (const char *msg);

#endif /*GRAPHICS_H*/
