/*
 * person.c - characters of Peach Gardens
 * 
 * Copyright (C) 2005  Toni Ronkko
 * 
 * Peach Gardens is free software; you can redistribute it and/or modify 
 * it under the terms of the GNU General Public License as published by the
 * Free Software Foundation; either version 2 of the License, or (at your
 * option) any later version.
 *
 * Peach Gardens is distributed in the hope that it will be useful, but 
 * WITHOUT ANY WARRANTY; without even the implied warranty of 
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU 
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with Peach Gardens; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * 
 * Apr  4 2005, Toni Ronkko <toni at users.playstation2-linux.com>
 *
 * $Id: person.c,v 1.1 2005/06/22 13:10:35 tronkko Exp $
 *
 * $Log: person.c,v $
 * Revision 1.1  2005/06/22 13:10:35  tronkko
 * Initial revision
 *
 *
 */
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <assert.h>

#include "person.h"
#include "locate.h"
#include "image.h"
#include "graphics.h"
#include "debug.h"


/* has the module been initialized already? */
static int initialized = 0;

/* face bitmap */
static struct image faces;


/*
 * Get name of person as a character string.
 */
const char *
person_name(
    enum person_t who)
{
  assert (initialized);
  
  switch (who) {
  case Stan:     return "Stan";
  case Rosalila: return "Rosalila";
  case Anna:     return "Anna";
  case Elsa:     return "Elsa";
    
  case Nobody:
  default:
    abort_msg ("invalid person %d", (int) who);
  }
  /*NOTREACHED*/
}


/*
 * Draw person's face to (x1,y1)-(x2,y2).
 */
void
draw_face(
    enum person_t who, int x1, int y1, int x2, int y2)
{
  int sx, sy;
  assert (initialized);
  
  /* locate face within bitmap */
  switch (who) {
  case Stan:     sx = 3; sy = 0; break;
  case Elsa:     sx = 4; sy = 2; break;
  case Anna:     sx = 1; sy = 3; break;    
  case Rosalila: sx = 1; sy = 1; break;
    
  case Nobody:
  default:
    abort_msg ("missing face %d", (int) who);
  }
  
  /* paint face */
  sx = sx * 48;
  sy = sy * 48;
  bitblt (&faces, sx, sy, sx + 48, sy + 48, x1, y1, x2, y2);
}


/*
 * Initialize module by loading faces from disk.
 */
void
init_person (void)
{
  assert (!initialized);
  
  /* load faces */
  init_image (&faces, locate ("face.pcx"));
  
  initialized = 1;
}


/*
 * De-initialize module.
 */
void
done_person (void)
{
  assert (initialized);
  
  /* get rid of images */
  done_image (&faces);
  
  initialized = 0;
}
