/*
 * pgardens.c - Peach Gardens main program
 * 
 * Copyright (C) 2005  Toni Ronkko
 * 
 * Peach Gardens is free software; you can redistribute it and/or modify 
 * it under the terms of the GNU General Public License as published by the
 * Free Software Foundation; either version 2 of the License, or (at your
 * option) any later version.
 *
 * Peach Gardens is distributed in the hope that it will be useful, but 
 * WITHOUT ANY WARRANTY; without even the implied warranty of 
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU 
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with Peach Gardens; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * 
 * Jun 12 2004, Toni Ronkko <toni at users.playstation2-linux.com>
 *
 * $Id: pgardens.c,v 1.1 2005/06/22 13:10:37 tronkko Exp $
 *
 * $Log: pgardens.c,v $
 * Revision 1.1  2005/06/22 13:10:37  tronkko
 * Initial revision
 *
 *
 */
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <assert.h>
#include <math.h>

#include "graphics.h"
#include "options.h"
#include "input.h"
#include "ticks.h"
#include "say.h"
#include "credits.h"
#include "view.h"
#include "map.h"
#include "spotlight.h"
#include "birds.h"
#include "clouds.h"
#include "stan.h"
#include "resource.h"
#include "image.h"
#include "text.h"
#include "locate.h"
#include "font10x16.h"
#include "font16x24.h"
#include "scriptfont.h"
#include "person.h"
#include "sprite.h"
#include "depth.h"
#include "title.h"
#include "debug.h"


/*
 * Frame of animation.  Multiple frames make up an animation sequence.
 */
struct frame
{
  int name;                                    /* frame id */
  int next;                                    /* next frame */
  int spr;                                     /* name of sprite */
  int len;                                     /* frame length in ticks */
};


struct stan
{
  float x, y;
  float dx, dy;
  float speed;
  float scale;
  int phase;
  int dir;
  ticks_t last_animated;
  ticks_t last_moved;
};

void animate_stan (struct stan *sp);

void draw_stan (struct view *vp, struct stan *sp);
  

#define WALKE0  0
#define WALKE1  1
#define WALKE2  2
#define WALKE3  3
#define WALKE4  4
#define WALKE5  5
#define WALKE6  6
#define WALKE7  7
#define WALKS0  8
#define WALKS1  9
#define WALKS2 10
#define WALKS3 11
#define WALKS4 12
#define WALKS5 13
#define WALKS6 14
#define WALKS7 15
#define WALKW0 16
#define WALKW1 17
#define WALKW2 18
#define WALKW3 19
#define WALKW4 20
#define WALKW5 21
#define WALKW6 22
#define WALKW7 23

#define WALKN0 24
#define WALKN1 25
#define WALKN2 26
#define WALKN3 27
#define WALKN4 28
#define WALKN5 29
#define WALKN6 30
#define WALKN7 31

#define STANDN  32
#define STANDNE 33
#define STANDE  34
#define STANDSE 35
#define STANDS  36
#define STANDSW 37
#define STANDW  38
#define STANDNW 39

#define WALKNW0 40
#define WALKNW1 41
#define WALKNW2 42
#define WALKNW3 43
#define WALKNW4 44
#define WALKNW5 45
#define WALKNW6 46
#define WALKNW7 47

#define WALKNE0 48
#define WALKNE1 49
#define WALKNE2 50
#define WALKNE3 51
#define WALKNE4 52
#define WALKNE5 53
#define WALKNE6 54
#define WALKNE7 55

#define WALKSE0 56
#define WALKSE1 57
#define WALKSE2 58
#define WALKSE3 59
#define WALKSE4 60
#define WALKSE5 61
#define WALKSE6 62
#define WALKSE7 63

#define WALKSW0 64
#define WALKSW1 65
#define WALKSW2 66
#define WALKSW3 67
#define WALKSW4 68
#define WALKSW5 69
#define WALKSW6 70
#define WALKSW7 71

/* starting offsets of various animations in frames table */
#define FRAME_WALKE    0
#define FRAME_WALKS    8
#define FRAME_WALKW   16
#define FRAME_WALKN   24
#define FRAME_STANDN  32
#define FRAME_STANDNE 33
#define FRAME_STANDE  34
#define FRAME_STANDSE 35
#define FRAME_STANDS  36
#define FRAME_STANDSW 37
#define FRAME_STANDW  38
#define FRAME_STANDNW 39
#define FRAME_WALKNW  40
#define FRAME_WALKNE  48
#define FRAME_WALKSE  56
#define FRAME_WALKSW  64

#define FRAMES  72
static struct frame frames[FRAMES] =
{
  {  0,  1, WALKE0, 80 }, 
  {  1,  2, WALKE1, 80 }, 
  {  2,  3, WALKE2, 80 }, 
  {  3,  4, WALKE3, 80 }, 
  {  4,  5, WALKE4, 80 }, 
  {  5,  6, WALKE5, 80 }, 
  {  6,  7, WALKE6, 80 }, 
  {  7,  0, WALKE7, 80 },

  {  8,  9, WALKS0, 80 }, 
  {  9, 10, WALKS1, 80 }, 
  { 10, 11, WALKS2, 80 }, 
  { 11, 12, WALKS3, 80 }, 
  { 12, 13, WALKS4, 80 }, 
  { 13, 14, WALKS5, 80 }, 
  { 14, 15, WALKS6, 80 }, 
  { 15,  8, WALKS7, 80 },
  
  { 16, 17, WALKW0, 80 }, 
  { 17, 18, WALKW1, 80 }, 
  { 18, 19, WALKW2, 80 }, 
  { 19, 20, WALKW3, 80 }, 
  { 20, 21, WALKW4, 80 }, 
  { 21, 22, WALKW5, 80 }, 
  { 22, 23, WALKW6, 80 }, 
  { 23, 16, WALKW7, 80 },

  { 24, 25, WALKN0, 80 }, 
  { 25, 26, WALKN1, 80 }, 
  { 26, 27, WALKN2, 80 }, 
  { 27, 28, WALKN3, 80 }, 
  { 28, 29, WALKN4, 80 }, 
  { 29, 30, WALKN5, 80 }, 
  { 30, 31, WALKN6, 80 }, 
  { 31, 24, WALKN7, 80 },
  
  { 32, 32, STANDN,  20 }, 
  { 33, 33, STANDNE, 20 }, 
  { 34, 34, STANDE,  20 }, 
  { 35, 35, STANDSE, 20 }, 
  { 36, 36, STANDS,  20 }, 
  { 37, 37, STANDSW, 20 }, 
  { 38, 38, STANDW,  20 }, 
  { 39, 39, STANDNW, 20 },

  { 40, 41, WALKNW0, 80 }, 
  { 41, 42, WALKNW1, 80 }, 
  { 42, 43, WALKNW2, 80 }, 
  { 43, 44, WALKNW3, 80 }, 
  { 44, 45, WALKNW4, 80 }, 
  { 45, 46, WALKNW5, 80 }, 
  { 46, 47, WALKNW6, 80 }, 
  { 47, 40, WALKNW7, 80 },

  { 48, 49, WALKNE0, 80 }, 
  { 49, 50, WALKNE1, 80 }, 
  { 50, 51, WALKNE2, 80 }, 
  { 51, 52, WALKNE3, 80 }, 
  { 52, 53, WALKNE4, 80 }, 
  { 53, 54, WALKNE5, 80 }, 
  { 54, 55, WALKNE6, 80 }, 
  { 55, 48, WALKNE7, 80 },

  { 56, 57, WALKSE0, 80 }, 
  { 57, 58, WALKSE1, 80 }, 
  { 58, 59, WALKSE2, 80 }, 
  { 59, 60, WALKSE3, 80 }, 
  { 60, 61, WALKSE4, 80 }, 
  { 61, 62, WALKSE5, 80 }, 
  { 62, 63, WALKSE6, 80 }, 
  { 63, 56, WALKSE7, 80 },

  { 64, 65, WALKSW0, 80 }, 
  { 65, 66, WALKSW1, 80 }, 
  { 66, 67, WALKSW2, 80 }, 
  { 67, 68, WALKSW3, 80 }, 
  { 68, 69, WALKSW4, 80 }, 
  { 69, 70, WALKSW5, 80 }, 
  { 70, 71, WALKSW6, 80 }, 
  { 71, 64, WALKSW7, 80 }
};

#define SPRITES 72
static struct sprite walk[SPRITES] =
{
  { WALKE0, "stan_we3.pcx", -32,-58, 9,40, 46,55, NULL }, 
  { WALKE1, "stan_we5.pcx", -32,-58, 9,40, 51,55, NULL }, 
  { WALKE2, "stan_we4.pcx", -32,-58, 9,40, 51,55, NULL }, 
  { WALKE3, "stan_we6.pcx", -32,-58, 9,40, 44,55, NULL }, 
  { WALKE4, "stan_we7.pcx", -32,-58, 9,40, 42,55, NULL },
  { WALKE5, "stan_we0.pcx", -32,-58, 9,40, 55,55, NULL }, 
  { WALKE6, "stan_we1.pcx", -32,-58, 9,40, 48,55, NULL }, 
  { WALKE7, "stan_we2.pcx", -32,-58, 9,40, 44,55, NULL }, 
  
  { WALKS0, "stan_ws2.pcx", -32,-59, 18,40, 45,56, NULL }, 
  { WALKS1, "stan_ws3.pcx", -32,-59, 18,35, 45,49, NULL }, 
  { WALKS2, "stan_ws4.pcx", -32,-59, 18,35, 45,63, NULL }, 
  { WALKS3, "stan_ws5.pcx", -32,-59, 18,35, 45,63, NULL }, 
  { WALKS4, "stan_ws6.pcx", -32,-59, 18,35, 45,58, NULL },
  { WALKS5, "stan_ws7.pcx", -32,-59, 18,35, 45,68, NULL }, 
  { WALKS6, "stan_ws0.pcx", -32,-59, 18,35, 45,63, NULL }, 
  { WALKS7, "stan_ws1.pcx", -32,-59, 18,35, 45,62, NULL },

  { WALKW0, "stan_ww3.pcx", -32,-58, 20,40, 56,60, NULL }, 
  { WALKW1, "stan_ww4.pcx", -32,-58, 12,40, 56,60, NULL }, 
  { WALKW2, "stan_ww5.pcx", -32,-58, 14,40, 56,60, NULL }, 
  { WALKW3, "stan_ww6.pcx", -32,-58, 17,40, 56,60, NULL }, 
  { WALKW4, "stan_ww7.pcx", -32,-58, 21,40, 56,60, NULL }, 
  { WALKW5, "stan_ww0.pcx", -32,-58, 10,40, 56,60, NULL }, 
  { WALKW6, "stan_ww1.pcx", -32,-58, 16,40, 56,60, NULL },
  { WALKW7, "stan_ww2.pcx", -32,-58, 20,40, 56,60, NULL }, 
  
  { WALKN0, "stan_wn2.pcx", -32,-58, 16,35, 48,63, NULL }, 
  { WALKN1, "stan_wn3.pcx", -32,-58, 16,35, 48,63, NULL }, 
  { WALKN2, "stan_wn4.pcx", -32,-58, 16,35, 48,63, NULL }, 
  { WALKN3, "stan_wn5.pcx", -32,-58, 16,35, 48,63, NULL }, 
  { WALKN4, "stan_wn6.pcx", -32,-58, 16,35, 48,63, NULL }, 
  { WALKN5, "stan_wn7.pcx", -32,-58, 16,35, 48,63, NULL }, 
  { WALKN6, "stan_wn0.pcx", -32,-58, 16,35, 48,63, NULL }, 
  { WALKN7, "stan_wn1.pcx", -32,-58, 16,35, 48,63, NULL },

  { STANDN,  "stan_sn.pcx",  -32,-58, 16,38, 47,56, NULL }, 
  { STANDNE, "stan_sne.pcx", -32,-58, 18,40, 46,58, NULL }, 
  { STANDE,  "stan_se.pcx",  -32,-58, 20,44, 43,56, NULL }, 
  { STANDSE, "stan_sse.pcx", -32,-58, 18,36, 46,58, NULL }, 
  { STANDS,  "stan_ss.pcx",  -32,-62, 16,42, 48,61, NULL }, 
  { STANDSW, "stan_ssw.pcx", -32,-58, 20,38, 48,58, NULL }, 
  { STANDW,  "stan_sw.pcx",  -32,-58, 22,43, 44,57, NULL }, 
  { STANDNW, "stan_snw.pcx", -32,-58, 22,39, 44,55, NULL }, 

  { WALKNW0, "stan_wnw2.pcx", -32,-58, 22,55, 51,58, NULL }, 
  { WALKNW1, "stan_wnw3.pcx", -32,-58, 22,55, 51,58, NULL }, 
  { WALKNW2, "stan_wnw4.pcx", -32,-58, 22,55, 51,58, NULL }, 
  { WALKNW3, "stan_wnw5.pcx", -32,-58, 22,55, 51,58, NULL }, 
  { WALKNW4, "stan_wnw6.pcx", -32,-58, 22,55, 51,58, NULL }, 
  { WALKNW5, "stan_wnw7.pcx", -32,-58, 22,55, 51,58, NULL }, 
  { WALKNW6, "stan_wnw0.pcx", -32,-58, 22,55, 51,58, NULL }, 
  { WALKNW7, "stan_wnw1.pcx", -32,-58, 22,55, 51,58, NULL },

  { WALKNE0, "stan_wne2.pcx", -32,-58, 11,32, 48,63, NULL }, 
  { WALKNE1, "stan_wne3.pcx", -32,-58, 11,32, 48,63, NULL }, 
  { WALKNE2, "stan_wne4.pcx", -32,-58, 11,32, 48,63, NULL }, 
  { WALKNE3, "stan_wne5.pcx", -32,-58, 11,32, 48,63, NULL }, 
  { WALKNE4, "stan_wne6.pcx", -32,-58, 11,32, 48,63, NULL }, 
  { WALKNE5, "stan_wne7.pcx", -32,-58, 11,32, 48,63, NULL }, 
  { WALKNE6, "stan_wne0.pcx", -32,-58, 11,32, 48,63, NULL }, 
  { WALKNE7, "stan_wne1.pcx", -32,-58, 11,32, 48,63, NULL },

  { WALKSE0, "stan_wse2.pcx", -32,-58, 16,37, 42,56, NULL }, 
  { WALKSE1, "stan_wse3.pcx", -32,-58, 19,36, 39,57, NULL }, 
  { WALKSE2, "stan_wse4.pcx", -32,-58, 18,34, 43,62, NULL }, 
  { WALKSE3, "stan_wse5.pcx", -32,-58, 16,35, 42,60, NULL }, 
  { WALKSE4, "stan_wse6.pcx", -32,-58, 18,36, 41,46, NULL }, 
  { WALKSE5, "stan_wse7.pcx", -32,-58, 16,37, 45,56, NULL }, 
  { WALKSE6, "stan_wse0.pcx", -32,-58, 13,36, 53,57, NULL }, 
  { WALKSE7, "stan_wse1.pcx", -32,-58, 14,37, 46,56, NULL },

  { WALKSW0, "stan_wsw2.pcx", -32,-58, 26,42, 43,58, NULL }, 
  { WALKSW1, "stan_wsw3.pcx", -32,-58, 21,42, 45,58, NULL }, 
  { WALKSW2, "stan_wsw4.pcx", -32,-58, 16,40, 48,60, NULL }, 
  { WALKSW3, "stan_wsw5.pcx", -32,-58, 17,40, 46,58, NULL }, 
  { WALKSW4, "stan_wsw6.pcx", -32,-58, 23,41, 45,57, NULL }, 
  { WALKSW5, "stan_wsw7.pcx", -32,-58, 25,38, 39,58, NULL }, 
  { WALKSW6, "stan_wsw0.pcx", -32,-58, 20,38, 42,62, NULL }, 
  { WALKSW7, "stan_wsw1.pcx", -32,-58, 24,38, 40,60, NULL }
};

/* is module currently initialized? */
static int initialized = 0;


/*
 * Initialize Stan by loading image files from disk.
 */
void
init_stan (void)
{
  int i;
  assert (!initialized);

  /* load walk images */
  for (i = 0; i < SPRITES; i++) {
    assert (walk[i].name == i);
    init_spite (&walk[i]);    
  }
  
  initialized = 1;
}


void
draw_stan(
    struct view *vp, struct stan *sp)
{
  int i;
  struct sprite *spr;
  
  assert (vp != NULL);
  assert (sp != NULL);

  /* get sprite */
  i = sp->phase;
  assert (0 <= i  &&  i < FRAMES);
  assert (0 <= frames[i].spr  &&  frames[i].spr < SPRITES);
  spr = &walk[frames[i].spr];  
  
  /* draw stan */
  draw_sprite (vp, spr, sp->x, sp->y, sp->scale);
}


void
animate_stan(
    struct stan *sp)
{
  ticks_t now;
  int elapsed;
  assert (sp != NULL);

  /* animate */
  now = ticks ();
  if (sp->last_animated < now) {
    elapsed = now - sp->last_animated;
    if (elapsed < 500) {
      /* less than half a second since previous animation */
      struct frame *fp;

      /* get current frame */
      assert (0 <= sp->phase  &&  sp->phase < FRAMES);
      fp = &frames[sp->phase];
     
      /* move to next frame if enough time has passed */
      if (elapsed > fp->len) {
        sp->last_animated = now;
        sp->phase = fp->next;
      }
      
    } else {
      /* too much time passed */
      sp->last_animated = now;
    }
  }

  /* advance */
  if (sp->last_moved < now) {
    elapsed = now - sp->last_moved;
    if (elapsed < 1000) {
      sp->x += sp->dx * sp->speed * elapsed;
      sp->y += sp->dy * sp->speed * elapsed;
    }
  }
  sp->last_moved = now;
}


/*
 * Uninitialize Stan.  Use this very function for releasing graphics
 * resources when Stan is not in scene.
 */
void
done_stan (void)
{
  int i;
  assert (initialized);

  /* release all images */
  for (i = 0; i < SPRITES; i++) {
    if (walk[i].imp != NULL) {
      done_sprite (&walk[i]);
    }
  }
  
  initialized = 0;
}


int
main (int argc, char *argv[])
{
  
  /* read command line args */
  init_graphics ();
  get_defaults ();
  read_options (&argc, argv);

  /* initialize subsystems */
  init_resource ();
  init_input ();
  init_birds ();
  init_clouds ();
  init_title ();
  init_say ();
  init_font10x16 ();
  init_scriptfont ();
  init_font16x24 ();
  init_spotlight ();
  init_person ();
  
  /* initialize graphics device */
  if (open_screen ()) {

    struct view v;
    int done = 0;
    int ev;
    int dir;
    int x = 320, y = 256;
    int clouds_enabled = 0;
    int birds_enabled = 1;
    struct map *mp = hometown;
    struct stan st = {
      860, 1270,                               /* x, y */
      0, 1,                                    /* dx, dy */
      0.08,                                    /* speed */
      0.5,                                     /* scale */
      1,                                       /* phase */
      1,                                       /* dir */
      0,                                       /* when animated */
      0                                        /* when moved */
    };
    
    init_view (&v);
    init_stan ();
    center_view (&v, x, y);
    
    while (!done) {
      clear ();
      zreset ();
      center_view (&v, st.x, st.y);
      
      /* paint map */
      draw_map (&v, mp);
      
      /* draw Stan */
      draw_stan (&v, &st);
      
      /* draw all depth enabled objects */
      zdraw ();
      
      /* draw birds on the top of other graphic */
      if (birds_enabled) {
        draw_birds (&v);
      }
      if (clouds_enabled) {
        draw_clouds (&v);
      }
      
      // setfg (0, 0, 0xff);
      // fade (0, 0, screenwidth(), screenheight(), 0xb0);
      // setfg (0, 0, 0);
      // draw_spotlight (320, 256, 300, 200);
      
#if 0
      say (Rosalila, 
          "I cannot get any precious jewels from the mine "
          "because the mine is full of monsters. I need you to clear "
          "out the mine for me.");
#endif
      
      /* no more graphics follows */
      flush_gfx ();
      
      animate_clouds ();
      animate_stan (&st);
      animate_birds (&v);
      generate_birds (&v);
      
      /* handle one-shot events */
      ev = get_event ();
      while (ev != NOEVENT) {
        
        /* parse one event */
        switch (ev) {
        case ERASE:
          if (scale(&v) > 1) {
            scale_view (&v, 1);
            clouds_enabled = 0;
            birds_enabled = 1;
          } else if (scale(&v) > 0.5) {
            scale_view (&v, 0.5);
            clouds_enabled = 1;
            birds_enabled = 0;
          } else {
            scale_view (&v, 2);
            clouds_enabled = 0;
            birds_enabled = 1;
          }
          break;
          
        case EXIT:
          if (mp == hometown) {
            mp = mine;
            st.x = 220;
            st.y = 700;
            st.scale = 1.0;
            birds_enabled = 0;
          } else {
            done = 1;
          }
          break;
        }
        
        ev = get_event ();
      }
      
      dir = 0;
      if (pressed (RIGHT)) dir = dir + 1;
      if (pressed (LEFT )) dir = dir + 2;
      if (pressed (UP   )) dir = dir + 4;
      if (pressed (DOWN )) dir = dir + 8;
      switch (dir) {
        
      case 0: /* stopped */
        st.dx = 0;
        st.dy = 0;
        switch (st.dir) {
        case 0:   break;
        case 1:   st.phase = FRAME_STANDE;  break;
        case 2:   st.phase = FRAME_STANDW;  break;
        case 4:   st.phase = FRAME_STANDN;  break;
        case 4+1: st.phase = FRAME_STANDNE; break;
        case 4+2: st.phase = FRAME_STANDNW; break;
        case 8:   st.phase = FRAME_STANDS;  break;
        case 8+1: st.phase = FRAME_STANDSE; break;
        case 8+2: st.phase = FRAME_STANDSW; break;
        }
        break;
        
      case 1: /* east */
        st.dx = 1;
        st.dy = 0;
        if (st.dir != dir) {
          st.phase = FRAME_WALKE;
        }
        break;
        
      case 2: /* west */
        st.dx = -1;
        st.dy = 0;
        if (st.dir != dir) {
          st.phase = FRAME_WALKW;
        }
        break;
        
      case 4: /* north */            
        st.dx = 0;
        st.dy = -1;
        if (st.dir != dir) {
          st.phase = FRAME_WALKN;
        }
        break;
        
      case 4+1: /* north east */ 
        st.dx = 1;
        st.dy = -1;
        if (st.dir != dir) {
          st.phase = FRAME_WALKNE;
        }
        break;
        
      case 4+2: /* north west */
        st.dx = -1;
        st.dy = -1;
        if (st.dir != dir) {
          st.phase = FRAME_WALKNW;
        }
        break;
        
      case 8: /* south */
        st.dx = 0;
        st.dy = 1;
        if (st.dir != dir) {
          st.phase = FRAME_WALKS;
        }
        break;
        
      case 8+1: /* south east */
        st.dx = 1;
        st.dy = 1;
        if (st.dir != dir) {
          st.phase = FRAME_WALKSE;
        }
        break;
        
      case 8+2: /* south west */
        st.dx = -1;
        st.dy = 1;
        if (st.dir != dir) {
          st.phase = FRAME_WALKSW;
        }
        break;
        
      default:
        abort_msg ("invalid button combination");
      }
      st.dir = dir;
      
      swap_buffers ();
    }
    
    zreset ();
    done_view (&v);
    done_stan ();

    credits();
    
    close_screen ();
  } else {
    abort_msg ("cannot open display");
  }
  
  done_person ();
  done_font10x16 ();
  done_scriptfont ();
  done_font16x24 ();
  done_spotlight ();
  done_say ();
  done_title ();
  done_clouds ();  
  done_birds ();
  done_input ();
  done_resource ();
  done_graphics ();
  return EXIT_SUCCESS;
}
  
