/*
 * scriptfont.c - variable width hand-writing font
 * 
 * Copyright (C) 2005  Toni Ronkko
 * 
 * Peach Gardens is free software; you can redistribute it and/or modify 
 * it under the terms of the GNU General Public License as published by the
 * Free Software Foundation; either version 2 of the License, or (at your
 * option) any later version.
 *
 * Peach Gardens is distributed in the hope that it will be useful, but 
 * WITHOUT ANY WARRANTY; without even the implied warranty of 
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU 
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with Peach Gardens; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * 
 * Apr  3 2005, Toni Ronkko <Toni.Ronkko@uku.fi>
 *
 * $Id: scriptfont.c,v 1.1 2005/06/22 13:11:10 tronkko Exp $
 *
 * $Log: scriptfont.c,v $
 * Revision 1.1  2005/06/22 13:11:10  tronkko
 * Initial revision
 *
 *
 */
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <assert.h>

#include "scriptfont.h"
#include "locate.h"
#include "font.h"
#include "text.h"
#include "graphics.h"


/* publicly usable font structure */
struct font *scriptfont = NULL;
static struct font _scriptfont;

/* has module been initialized? */
static int initialized = 0;

/*
 * Character widths for hand-writing font.
 */
static const struct glyph glyphs[] = {

  /*code   x   y   w  h  spc    asc desc */
  { ' ',   0,  0,  0, 0, 0,  16, 20, 6 },
  
  { 'A',   0,  0, 35,26, 0,  -4, 20, 6 },
  { 'B',  35,  0, 28,26, 0,   0, 20, 6 },
  { 'C',  63,  0, 19,26, 0,   0, 20, 6 },
  { 'D',  82,  0, 27,26, 0,   0, 20, 6 },
  { 'E', 109,  0, 19,26, 0,   0, 20, 6 },
  { 'F', 128,  0, 30,26, 0,  -4, 20, 6 },
  { 'G', 158,  0, 33,26, 0,  -2, 20, 6 },
  { 'H', 191,  0, 38,26, 0, -10, 20, 6 },
  { 'I', 229,  0, 27,26, 0,   0, 20, 6 },

  { 'J',   0, 26, 32,26,-6,  -8, 20, 6 },
  { 'K',  32, 26, 34,26, 0,  -4, 20, 6 },
  { 'L',  66, 26, 27,26, 0,   0, 20, 6 },
  { 'M',  93, 26, 39,26, 0,  -4, 20, 6 },
  { 'N', 132, 26, 34,26, 0,  -8, 20, 6 },
  { 'O', 166, 26, 20,26, 0,   0, 20, 6 },
  { 'P', 186, 26, 28,26, 0,   0, 20, 6 },
  { 'Q', 214, 26, 21,26, 0,   0, 20, 6 },

  { 'R',   0, 52, 28,26, 0,   0, 20, 6 },
  { 'S',  28, 52, 20,26, 0,   0, 20, 6 },
  { 'T',  48, 52, 30,26, 0,  -8, 20, 6 },
  { 'U',  78, 52, 27,26, 0,   0, 20, 6 },
  { 'V', 106, 52, 23,26, 0,   0, 20, 6 },
  { 'W', 129, 52, 30,26, 0,  -4, 20, 6 },
  { 'X', 159, 52, 31,26, 0,   0, 20, 6 },
  { 'Y', 190, 52, 30,26, 0,  -6, 20, 6 },
  { 'Z', 220, 52, 30,26, 0,   0, 20, 6 },

  { 'a',   0, 78, 17,26, 0,  -1, 20, 6 },
  { 'b',  17, 78, 14,26,-1,   2, 20, 6 },
  { 'c',  31, 78, 14,26, 0,   0, 20, 6 },
  { 'd',  48, 78, 19,26, 0,   0, 20, 6 },
  { 'e',  67, 78, 14,26, 0,   0, 20, 6 },
  { 'f',  81, 78, 24,26,-8,  -5, 20, 6 },
  { 'g', 105, 78, 18,26,-1,   0, 20, 6 },
  { 'h', 123, 78, 17,26,-4,   0, 20, 6 },
  { 'i', 140, 78, 10,26, 0,   0, 20, 6 },
  { 'j', 150, 78, 19,26,-8,  -2, 20, 6 },
  { 'k', 169, 78, 17,26,-2,   0, 20, 6 },
  { 'l', 186, 78, 11,26, 0,  -2, 20, 6 },
  { 'm', 197, 78, 24,26,-2,   0, 20, 6 },
  { 'n', 221, 78, 18,26,-2,   0, 20, 6 },
  { 'o', 239, 78, 12,26, 0,   0, 20, 6 },

  { 'p',   0,104, 20,26,-6,   0, 20, 6 },
  { 'q',  21,104, 14,26, 0,   0, 20, 6 },
  { 'r',  35,104,  9,26, 2,   0, 20, 6 },
  { 's',  44,104,  9,26, 0,   4, 20, 6 },
  { 't',  53,104, 11,26, 2,   0, 20, 6 },
  { 'u',  64,104, 16,26, 0,   0, 20, 6 },
  { 'v',  80,104, 12,26, 0,  -2, 20, 6 },
  { 'w',  92,104, 18,26, 0,   0, 20, 6 },
  { 'x', 110,104, 14,26, 0,   0, 20, 6 },
  { 'y', 124,104, 17,26,-2,   0, 20, 6 },
  { 'z', 141,104, 13,26, 0,   0, 20, 6 },

  { '0',   0,130, 18,26, 0,   0, 20, 6 },
  { '1',  18,130, 16,26, 0,  -2, 20, 6 },
  { '2',  34,130, 21,26, 0,   0, 20, 6 },
  { '3',  55,130, 19,26, 0,   0, 20, 6 },
  { '4',  74,130, 19,26, 0,  -2, 20, 6 },
  { '5',  93,130, 21,26, 0,   0, 20, 6 },
  { '6', 114,130, 17,26, 0,   0, 20, 6 },
  { '7', 131,130, 16,26, 0,  -2, 20, 6 },
  { '8', 147,130, 18,26, 0,   0, 20, 6 },
  { '9', 165,130, 17,26, 0,   0, 20, 6 },
  { '?', 182,130, 18,26, 0,   0, 20, 6 },
  { '!', 200,130, 15,26, 0,  -2, 20, 6 },
  { '.', 215,130,  5,26, 2,   4, 20, 6 },
  { ',', 220,130,  5,26, 2,   2, 20, 6 },
  { ':', 225,139,  9,11, 2,   2, 11, 6 },
  { '/', 235,130, 18,20, 0,  -8, 20, 6 },
  
  { '\0',  0,  0,  0, 0, 0,   0,  0, 0 } 
};


/*
 * Load font to memory.
 */
void
init_scriptfont (void)
{
  assert (!initialized);
  
  /* set up structure */
  init_bitmap_font (&_scriptfont, locate("alison.pcx"), glyphs);
  scriptfont = &_scriptfont;
  
  initialized = 1;
}


/*
 * Unload font from memory.
 */
void
done_scriptfont (void)
{
  assert (initialized);
  
  assert (scriptfont == &_scriptfont);
  done_font (&_scriptfont);
  scriptfont = NULL;
  
  initialized = 0;
}
