/*
 * sprite.c - sprite functions
 * 
 * Copyright (C) 2005  Toni Ronkko
 * 
 * Peach Gardens is free software; you can redistribute it and/or modify 
 * it under the terms of the GNU General Public License as published by the
 * Free Software Foundation; either version 2 of the License, or (at your
 * option) any later version.
 *
 * Peach Gardens is distributed in the hope that it will be useful, but 
 * WITHOUT ANY WARRANTY; without even the implied warranty of 
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU 
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with Peach Gardens; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * 
 * Feb  3 2005, Toni Ronkko <toni at users.playstation2-linux.com>
 *
 * $Id: sprite.c,v 1.1 2005/04/03 16:25:34 tronkko Exp $
 *
 * $Log: sprite.c,v $
 * Revision 1.1  2005/04/03 16:25:34  tronkko
 * Initial revision
 *
 *
 */
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <assert.h>

#include "sprite.h"
#include "image.h"
#include "depth.h"
#include "locate.h"
#include "view.h"
#include "debug.h"


/*
 * Initialize sprite structure by loading image file into memory.
 */
void
init_spite(
    struct sprite *spr)
{
  struct image *imp;
  assert (spr != NULL);
  assert (spr->imp == NULL);
  
  /* allocate memory for the image */
  imp = malloc (sizeof (struct image));
  DEBUG (("%p: alloc sprite", imp));
  if (imp != NULL) {
    /* load image to memory */
    init_image (imp, locate (spr->filename));
    
    /* assign image to sprite */
    spr->imp = imp;
  } else {
    abort_msg ("out of memory");
  }
}


/*
 * Release sprite from memory.  Note that the function must not destroy
 * member variables, as sprites released by the function may be
 * re-initialized later.
 */
void
done_sprite(
    struct sprite *spr)
{
  assert (spr != NULL);
  assert (spr->imp != NULL);
  
  /* release image */
  done_image (spr->imp);
  DEBUG (("%p: free sprite", spr->imp));
  free (spr->imp);
  spr->imp = NULL;
}


/*
 * Draw sprite with depth cue.  That is, sprite will be queued for
 * drawing later by zdraw().
 *
 * The sprite must not be un-initialized prior to calling zdraw() or
 * zreset().
 */
void
draw_sprite(
    struct view *vp, struct sprite *spr, int x, int y, float ss)
{
  int x1, y1, x2, y2;
  float vs;
  int w, h, z;
  
  assert (vp != NULL);
  assert (spr != NULL);
  assert (spr->imp != NULL);

  /* fetch scale and image dimensions to temporary variables */
  vs = scale(vp);
  w = spr->imp->width;
  h = spr->imp->height;
  
  /*
   * Compute location of sprite on screen.  Notice how scaling changes the
   * location of the upper left corner.  This behaviour is meant to keep
   * hotspot properly aligned with the rest of the screen when a sprite is
   * scaled.
   */
  x1 = (x + spr->xoffs * ss - viewx(vp)) * vs;
  x2 = x1 + w * vs * ss;
  y2 = (y + spr->yoffs - viewy(vp)) * vs + h * vs;
  y1 = y2 - h * vs * ss;

  /* compute depth coordinate */
  z = (y - viewy(vp)) * vs;
  
  /* queue image for drawing */
  zbitblt (spr->imp, y2, 0,0,w,h, x1,y1,x2,y2);
}


