/*
 * sprite.h - sprite functions
 * 
 * Copyright (C) 2005  Toni Ronkko
 * 
 * Peach Gardens is free software; you can redistribute it and/or modify 
 * it under the terms of the GNU General Public License as published by the
 * Free Software Foundation; either version 2 of the License, or (at your
 * option) any later version.
 *
 * Peach Gardens is distributed in the hope that it will be useful, but 
 * WITHOUT ANY WARRANTY; without even the implied warranty of 
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU 
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with Peach Gardens; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * 
 * Feb  3 2005, Toni Ronkko <toni at users.playstation2-linux.com>
 *
 * $Id: sprite.h,v 1.1 2005/04/03 16:33:15 tronkko Exp $
 *
 * $Log: sprite.h,v $
 * Revision 1.1  2005/04/03 16:33:15  tronkko
 * Initial revision
 *
 *
 */
#ifndef SPRITE_H
#define SPRITE_H

/* forward-decl */
struct view;

/* max length of sprite filename without path */
#define MAX_SPRITE_NAME 20


/*
 * An image with hotspot and foothold.  Hotspots are used to align frames of
 * animation correctly.  This is especially important if frames are of
 * different size.  Foothold is used for hit testing.  Basically, foothold
 * area is used to keep the sprite away from walls and other objects.  In
 * any event foothold areas of two sprites may not overlap.
 */
struct sprite
{
  int name;                                    /* name in memory */
  char filename[MAX_SPRITE_NAME];              /* name in disk */
  int xoffs, yoffs;                            /* hot spot */
  int x1, y1, x2, y2;                          /* foothold */
  struct image *imp;
};


/* initialize */
void init_spite (struct sprite *spr);
void done_sprite (struct sprite *spr);

/* draw sprite at (x,y) in scale ss */
void draw_sprite(
    struct view *vp, struct sprite *spr, int x, int y, float ss);

#endif /*SPRITE_H*/
