/*
 * t-menu.c - check main menu
 * 
 * Peach Gardens is free software; you can redistribute it and/or modify 
 * it under the terms of the GNU General Public License as published by the
 * Free Software Foundation; either version 2 of the License, or (at your
 * option) any later version.
 *
 * Peach Gardens is distributed in the hope that it will be useful, but 
 * WITHOUT ANY WARRANTY; without even the implied warranty of 
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU 
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with Peach Gardens; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * 
 * Mar  9 2005, Toni Ronkko <toni at users.playstation2-linux.com>
 *
 * $Id: t-menu.c,v 1.2 2005/06/22 13:15:00 tronkko Exp $
 *
 * $Log: t-menu.c,v $
 * Revision 1.2  2005/06/22 13:15:00  tronkko
 * moved menu to a distinct module
 *
 * Revision 1.1  2005/04/03 16:35:27  tronkko
 * Initial revision
 *
 *
 */
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <assert.h>
#include <math.h>

#include "mainmenu.h"
#include "input.h"
#include "title.h"
#include "graphics.h"
#include "options.h"
#include "text.h"
#include "font16x24.h"
#include "font10x16.h"
#include "scriptfont.h"
#include "locate.h"
#include "credits.h"
#include "chapter.h"
#include "debug.h"

#undef NDEBUG
#include <assert.h>



int
main (int argc, char *argv[])
{
  int chatty = 0;
  (void) argv;
  if (argc > 1) chatty = 1;

  /* read command line args */
  init_graphics ();
  get_defaults ();
  read_options (&argc, argv);
  init_input ();
  
  /* load resources */
  init_font10x16 ();
  init_scriptfont ();
  init_font16x24 ();
  init_title ();
  
  /* show menu */
  if (chatty) {
    int ok;
    int done;
    
    /* open graphics console */
    ok = open_screen ();
    assert (ok);
    
    /* show menu */
    done = 0;
    while (!done) {
      int option = mainmenu ();
      switch (option) {
      case CANCEL_MENU:
        break;
        
      case EXIT_GAME:
        done = 1;
        break;
        
      case NEW_GAME:
        chapter_title (1, "storm.pcx", "Birthday Party");
        break;
        
      case LOAD_GAME:
        break;
        
      case OPTIONS:
        break;
        
      case COPYRIGHT:
        copyright ();
        break;
        
      case CREDITS:
        credits ();
        break;
        
      default:
        /* invalid menu option */
        abort ();
      }
    }
        
    close_screen ();
  }
  
  /* de-initialize */
  done_title ();
  done_font16x24 ();
  done_scriptfont ();
  done_font10x16 ();
  done_input ();
  done_graphics ();
  return EXIT_SUCCESS;
}
