/*
 * t-script.c - check scripting features
 * 
 * Peach Gardens is free software; you can redistribute it and/or modify 
 * it under the terms of the GNU General Public License as published by the
 * Free Software Foundation; either version 2 of the License, or (at your
 * option) any later version.
 *
 * Peach Gardens is distributed in the hope that it will be useful, but 
 * WITHOUT ANY WARRANTY; without even the implied warranty of 
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU 
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with Peach Gardens; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * 
 * Apr  4 2005, Toni Ronkko <toni at users.playstation2-linux.com>
 *
 * $Id: t-script.c,v 1.1 2005/06/22 13:16:03 tronkko Exp $
 *
 * $Log: t-script.c,v $
 * Revision 1.1  2005/06/22 13:16:03  tronkko
 * Initial revision
 *
 *
 */
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <assert.h>

#include "script.h"
#include "options.h"
#include "person.h"
#include "graphics.h"

#undef NDEBUG
#include <assert.h>

/* play script to stdin/stdout for debugging */
static int debug_script (struct script *sp);


int
main (int argc, char *argv[])
{
  int chatty = 0;
  (void) argv;
  if (argc > 1) chatty = 1;

  /* init graphics for image registration */
  init_graphics ();
  get_defaults ();
  read_options (&argc, argv);

  /* init modules */
  init_script ();
  init_person ();

  /* test a script */
  if (chatty) {
    /* get script id from command line */
    int id = atoi (argv[1]);
    struct script *sp = find_script (id);
    if (sp != NULL) {

      /* "play" script in console mode */
      debug_script (sp);
      
    } else {
      fprintf (stderr, "missing script %d\n", id);
      return EXIT_FAILURE;
    }
  }

  done_person ();
  done_script ();
  done_graphics ();
  return EXIT_SUCCESS;
}


/*
 * Play script to stdout and ask questions from stdin.  This is solely
 * for debugging conversation scripts without going in to graphics mode.
 */
static int
debug_script(
    struct script *sp)
{
  struct phrase *pp;
  int retcode = -1;
  int answer = '\0';
  assert (sp != NULL  &&  sp->no >= 0);

  /* parse script */
  pp = sp->phrases;
  while (pp != NULL) {

    /* execute phrase */
    switch (pp->type) {
    case Say:
      assert (pp->what != NULL);
      printf ("%s: %s\n", person_name (pp->who), pp->what);
      fflush (stdout);
      pp = pp->next;
      break;

    case Goto:
      pp = find_phrase (sp, pp->target);
      assert (pp != NULL);
      break;

    case Ask:
      assert (pp->what != NULL);
      printf ("%s: %s [y/n]\n", person_name (pp->who), pp->what);
      fflush (stdout);
      pp = pp->next;
      answer = fgetc (stdin);
      while (answer != EOF  &&  answer != 'y'  &&  answer != 'n') {
        fflush (stdout);
        answer = fgetc (stdin);
      }
      break;

    case IfNo:
      if (answer == 'n') {
        pp = find_phrase (sp, pp->target);
        assert (pp != NULL);
      } else {
        pp = pp->next;
      }
      break;
      
    case IfYes:
      if (answer == 'y') {
        pp = find_phrase (sp, pp->target);
        assert (pp != NULL);
      } else {
        pp = pp->next;
      }
      break;

    case Exit:
      retcode = pp->target;
      pp = NULL;
      break;
      
    default:
      /* invalid phrase */
      abort ();
    }
  }
  printf ("Exit %d\n", retcode);
  return retcode;
}
