/*
 * t-split.cpp - check splitting of strings
 * 
 * Copyright (C) 2005  Toni Ronkko
 * 
 * Peach Gardens is free software; you can redistribute it and/or modify 
 * it under the terms of the GNU General Public License as published by the
 * Free Software Foundation; either version 2 of the License, or (at your
 * option) any later version.
 *
 * Peach Gardens is distributed in the hope that it will be useful, but 
 * WITHOUT ANY WARRANTY; without even the implied warranty of 
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU 
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with Peach Gardens; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 * 
 * 
 * Dec 29 2003, Toni Ronkko <toni at users.playstation2-linux.com>
 *
 * $Id: t-split.c,v 1.2 2005/01/09 21:16:03 tronkko Exp $
 *
 * $Log: t-split.c,v $
 * Revision 1.2  2005/01/09 21:16:03  tronkko
 * update copyright
 *
 * Revision 1.1  2004/11/20 10:58:52  tronkko
 * Initial revision
 *
 *
 */
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <assert.h>

#include "split.h"

#undef NDEBUG
#include <assert.h>



int
main (int argc, char *argv[])
{
  int chatty = 0;
  (void) argv;
  if (argc > 1) chatty = 1;

  /* basic splitting */
  {
    char **av;
    int ac;
    int ok;

    ok = split (&ac, &av, "  1  2 3  ", " \t", 1);
    assert (ok);
    assert (ac == 3);
    assert (strcmp (av[0], "1") == 0);
    assert (strcmp (av[1], "2") == 0);
    assert (strcmp (av[2], "3") == 0);
    
    free (av);
  }

  /* custom separators */
  {
    char **av;
    int ac;
    int ok;
    
    ok = split (&ac, &av, ":path1:path2:path3:", ":", 1);
    assert (ok);
    assert (ac == 3);
    assert (strcmp (av[0], "path1") == 0);
    assert (strcmp (av[1], "path2") == 0);
    assert (strcmp (av[2], "path3") == 0);

    free (av);
  }

  /* custom separators, do not skip multiple separators */
  {
    char **av;
    int ac;
    int ok;
    
    ok = split (&ac, &av, ":path1:path2:path3:", ":", 0);
    assert (ok);
    assert (ac == 5);
    assert (strcmp (av[0], "") == 0);
    assert (strcmp (av[1], "path1") == 0);
    assert (strcmp (av[2], "path2") == 0);
    assert (strcmp (av[3], "path3") == 0);
    assert (strcmp (av[4], "") == 0);
    
    free (av);
  }
  
  /* escape characters */
  {
    char **av;
    int ac;
    int ok;
    
    ok = split (&ac, &av, "\\a \\b \\r", " \t", 1);
    assert (ok);
    assert (ac == 3);
    assert (strcmp (av[0], "\a") == 0);
    assert (strcmp (av[1], "\b") == 0);
    assert (strcmp (av[2], "\r") == 0);
    
    free (av);
  }
  
  /* quoted strings */
  {
    char **av;
    int ac;
    int ok;
    
    ok = split (&ac, &av, "' 1 a' \" ' \t\"", " \t", 1);
    assert (ok);
    assert (ac == 2);
    assert (strcmp (av[0], " 1 a") == 0);
    assert (strcmp (av[1], " ' \t") == 0);
    
    free (av);
  }

  /* nothing to split */
  {
    char **av;
    int ac;
    int ok;
    
    ok = split (&ac, &av, " \t ", " \t", 1);
    assert (ok);
    assert (ac == 0);
    
    ok = split (&ac, &av, "", " \t", 1);
    assert (ok);
    assert (ac == 0);
  }
  
  return EXIT_SUCCESS;
}
